@@ -16,21 +16,21 @@
 #define AR(str) rb_str_concat(buf, (str))
 
 #define A_INDENT add_indent(buf, indent)
-#define A_ID(id) add_id(buf, id)
-#define A_INT(val) rb_str_catf(buf, "%d", (val));
-#define A_LONG(val) rb_str_catf(buf, "%ld", (val));
+#define A_ID(id) add_id(buf, (id))
+#define A_INT(val) rb_str_catf(buf, "%d", (val))
+#define A_LONG(val) rb_str_catf(buf, "%ld", (val))
 #define A_LIT(lit) AR(rb_inspect(lit))
 #define A_NODE_HEADER(node) \
     rb_str_catf(buf, "@ %s (line: %d)", ruby_node_name(nd_type(node)), nd_line(node))
 #define A_FIELD_HEADER(name) \
-    rb_str_catf(buf, "+- %s:", name)
+    rb_str_catf(buf, "+- %s:", (name))
 
 #define D_NULL_NODE A_INDENT; A("(null node)"); A("\n");
 #define D_NODE_HEADER(node) A_INDENT; A_NODE_HEADER(node); A("\n");
 
 #define COMPOUND_FIELD(name, name2, block) \
     do { \
-	A_INDENT; A_FIELD_HEADER(comment ? name2 : name); A("\n"); \
+	A_INDENT; A_FIELD_HEADER(comment ? (name2) : (name)); A("\n"); \
 	rb_str_cat2(indent, next_indent); \
 	block; \
 	rb_str_resize(indent, RSTRING_LEN(indent) - 4); \
@@ -38,7 +38,7 @@
 
 #define SIMPLE_FIELD(name, name2, block) \
     do { \
-	A_INDENT; A_FIELD_HEADER(comment ? name2 : name); A(" "); block; A("\n"); \
+	A_INDENT; A_FIELD_HEADER(comment ? (name2) : (name)); A(" "); block; A("\n"); \
     } while (0)
 
 #define F_CUSTOM1(name, ann, block) SIMPLE_FIELD(#name, #name " (" ann ")", block)
@@ -77,7 +77,7 @@ add_id(VALUE buf, ID id)
     else {
 	VALUE str = rb_id2str(id);
 	if (str) {
-	    A(":"); AR(rb_id2str(id));
+	    A(":"); AR(str);
 	}
 	else {
 	    A("(internal variable)");
@@ -501,7 +501,7 @@ dump_node(VALUE buf, VALUE indent, int comment, NODE *node)
 	F_CUSTOM1(nd_nth, "variable", {
 	   char name[3];
 	   name[0] = '$';
-	   name[1] = node->nd_nth;
+	   name[1] = (char)node->nd_nth;
 	   name[2] = '\0';
 	   A(name);
 	});
@@ -823,6 +823,15 @@ dump_node(VALUE buf, VALUE indent, int comment, NODE *node)
 	F_NODE(nd_next, "next");
 	break;
 
+      case NODE_KW_ARG:
+	ANN("keyword arguments");
+	ANN("format: def method_name([nd_body=some], [nd_next..])");
+	ANN("example: def foo(a:1, b:2); end");
+	F_NODE(nd_body, "body");
+	LAST_NODE;
+	F_NODE(nd_next, "next");
+	break;
+
       case NODE_POSTARG:
 	ANN("post arguments");
 	ANN("format: *[nd_1st], [nd_2nd..] = ..");
@@ -837,52 +846,21 @@ dump_node(VALUE buf, VALUE indent, int comment, NODE *node)
 	F_NODE(nd_2nd, "post arguments");
 	break;
 
-      case NODE_ARGS_AUX:
-	ANN("method parameters (cont'd)");
-	F_CUSTOM1(nd_rest, "rest argument", {
-	    if (node->nd_rest == 1) A("nil (with last comma)");
-	    else A_ID(node->nd_rest);
-	});
-	F_CUSTOM1(nd_body, "block argument", { A_ID((ID)node->nd_body); });
-	LAST_NODE;
-	F_CUSTOM2(nd_next, "aux info 2", {
-	    node = node->nd_next;
-	    next_indent = "|    ";
-	    if (!node) {
-		D_NULL_NODE;
-	    }
-	    else {
-		D_NODE_HEADER(node);
-		ANN("method parameters (cont'd)");
-		F_ID(nd_pid, "first post argument");
-		F_LONG(nd_plen, "post argument length");
-		LAST_NODE;
-		F_CUSTOM2(nd_next, "aux info 3", {
-		    node = node->nd_next;
-		    next_indent = "|   ";
-		    if (!node) {
-			D_NULL_NODE;
-		    }
-		    else {
-			D_NODE_HEADER(node);
-			ANN("method parameters (cont'd)");
-			F_NODE(nd_1st, "init arguments (m)");
-			LAST_NODE;
-			F_NODE(nd_2nd, "init arguments (p)");
-		    }
-		});
-	    }
-	});
-	break;
-
       case NODE_ARGS:
 	ANN("method parameters");
 	ANN("format: def method_name(.., [nd_opt=some], *[nd_rest], [nd_pid], .., &[nd_body])");
 	ANN("example: def foo(a, b, opt1=1, opt2=2, *rest, y, z, &blk); end");
-	F_LONG(nd_frml, "argc");
-	F_NODE(nd_next, "aux info 1");
+	F_INT(nd_ainfo->pre_args_num, "count of mandatory (pre-)arguments");
+	F_NODE(nd_ainfo->pre_init, "initialization of (pre-)arguments");
+	F_INT(nd_ainfo->post_args_num, "count of mandatory post-arguments");
+	F_NODE(nd_ainfo->post_init, "initialization of post-arguments");
+	F_ID(nd_ainfo->first_post_arg, "first post argument");
+	F_ID(nd_ainfo->rest_arg, "rest argument");
+	F_ID(nd_ainfo->block_arg, "block argument");
+	F_NODE(nd_ainfo->opt_args, "optional arguments");
 	LAST_NODE;
-	F_NODE(nd_opt, "optional arguments");
+	F_NODE(nd_ainfo->kw_args, "keyword arguments");
+	F_NODE(nd_ainfo->kw_rest_arg, "keyword rest argument");
 	break;
 
       case NODE_SCOPE:
