@@ -88,10 +88,6 @@ class TestPathname < Test::Unit::TestCase
     defassert(:cleanpath_aggressive, '/',       '///a/../..')
   end
 
-  if DOSISH
-    defassert(:cleanpath_aggressive, 'c:/foo/bar', 'c:\\foo\\bar')
-  end
-
   def cleanpath_conservative(path)
     Pathname.new(path).cleanpath(true).to_s
   end
@@ -128,10 +124,6 @@ class TestPathname < Test::Unit::TestCase
   defassert(:cleanpath_conservative, '/a',     '/../.././../a')
   defassert(:cleanpath_conservative, 'a/b/../../../../c/../d', 'a/b/../../../../c/../d')
 
-  if DOSISH
-    defassert(:cleanpath_conservative, 'c:/foo/bar', 'c:\\foo\\bar')
-  end
-
   if DOSISH_UNC
     defassert(:cleanpath_conservative, '//',     '//')
   else
@@ -724,6 +716,22 @@ class TestPathname < Test::Unit::TestCase
     }
   end
 
+  def test_write
+    with_tmpchdir('rubytest-pathname') {|dir|
+      path = Pathname("a")
+      path.write "abc"
+      assert_equal("abc", path.read)
+    }
+  end
+
+  def test_binwrite
+    with_tmpchdir('rubytest-pathname') {|dir|
+      path = Pathname("a")
+      path.binwrite "abc\x80"
+      assert_equal("abc\x80".b, path.binread)
+    }
+  end
+
   def test_sysopen
     with_tmpchdir('rubytest-pathname') {|dir|
       open("a", "w") {|f| f.write "abc" }
@@ -1306,17 +1314,4 @@ class TestPathname < Test::Unit::TestCase
       assert_equal("foo/bar", File.join(Pathname.new("foo"), Pathname.new("bar").taint))
     }.call
   end
-
-  def test_relative_path_from_casefold
-    assert_separately([], <<-'end;') #    do
-      module File::Constants
-        remove_const :FNM_SYSCASE
-        FNM_SYSCASE = FNM_CASEFOLD
-      end
-      require 'pathname'
-      foo = Pathname.new("fo\u{f6}")
-      bar = Pathname.new("b\u{e4}r".encode("ISO-8859-1"))
-      assert_instance_of(Pathname, foo.relative_path_from(bar))
-    end;
-  end
 end
